@@ -1,7 +1,7 @@
 
 /*
  *  autogen.c
- *  $Id: autogen.c,v 4.5 2005/01/26 01:10:55 bkorb Exp $
+ *  $Id: autogen.c,v 4.8 2005/06/07 22:26:43 bkorb Exp $
  *  This is the main routine for autogen.
  */
 
@@ -21,8 +21,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with AutoGen.  See the file "COPYING".  If not,
  *  write to:  The Free Software Foundation, Inc.,
- *             59 Temple Place - Suite 330,
- *             Boston,  MA  02111-1307, USA.
+ *             51 Franklin Street, Fifth Floor,
+ *             Boston, MA  02110-1301, USA.
  */
 
 tSCC zClientInput[] = "client-input";
@@ -202,6 +202,12 @@ doneCheck( void )
         "Scheme evaluation error.  AutoGen ABEND-ing in template\n"
         "\t%s on line %d\n";
 
+#ifdef SHELL_ENABLED
+    ag_scm_c_eval_string_from_file_line(
+        "(if (> (string-length shell-cleanup) 0)"
+        "    (shell shell-cleanup) )", __FILE__, __LINE__ );
+#endif
+
     fflush( stdout );
     fflush( stderr );
 
