@@ -1,12 +1,12 @@
 
 /*
- *  $Id: funcCase.c,v 1.49 2004/02/02 03:31:50 bkorb Exp $
+ *  $Id: funcCase.c,v 4.4 2005/06/07 22:26:43 bkorb Exp $
  *
  *  This module implements the CASE text function.
  */
 /*=--subblock=exparg=arg_name,arg_desc,arg_optional,arg_list=*/
 /*
- *  AutoGen copyright 1992-2004 Bruce Korb
+ *  AutoGen copyright 1992-2005 Bruce Korb
  *
  *  AutoGen is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -21,8 +21,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with AutoGen.  See the file "COPYING".  If not,
  *  write to:  The Free Software Foundation, Inc.,
- *             59 Temple Place - Suite 330,
- *             Boston,  MA  02111-1307, USA.
+ *             51 Franklin Street, Fifth Floor,
+ *             Boston, MA  02110-1301, USA.
  */
 
 #undef  IS_LOW
@@ -40,11 +40,8 @@
 
 tSCC zBadRe[]  = "Invalid regular expression:  error %d (%s):\n%s";
 
-STATIC void     compile_re( regex_t* pRe, char* pzPat, int flags );
-STATIC void     upString( char* pz );
-
 typedef tSuccess (tSelectProc)( char* pzText, char* pzMatch );
-STATIC tSelectProc
+static tSelectProc
     Select_Compare,
     Select_Compare_End,
     Select_Compare_Start,
@@ -59,9 +56,84 @@ STATIC tSelectProc
     Select_Match_Full,
     Select_Match_Always;
 
+/*
+ *  This is global data used to keep track of the current CASE
+ *  statement being processed.  When CASE statements nest,
+ *  these data are copied onto the stack and restored when
+ *  the nested CASE statement's ESAC function is found.
+ */
+typedef struct case_stack tCaseStack;
+struct case_stack {
+    tMacro*  pCase;
+    tMacro*  pSelect;
+};
+
+static tCaseStack  current_case;
+static tLoadProc mLoad_Select;
+
+static tpLoadProc apCaseLoad[ FUNC_CT ]   = { NULL };
+static tpLoadProc apSelectOnly[ FUNC_CT ] = { NULL };
+
+/* = = = START-STATIC-FORWARD = = = */
+/* static forward declarations maintained by :mkfwd */
+static void
+compile_re( regex_t* pRe, char* pzPat, int flags );
+
+static void
+upString( char* pz );
+
+static tSuccess
+Select_Compare( char* pzText, char* pzMatch );
+
+static tSuccess
+Select_Compare_End( char* pzText, char* pzMatch );
+
+static tSuccess
+Select_Compare_Start( char* pzText, char* pzMatch );
+
+static tSuccess
+Select_Compare_Full( char* pzText, char* pzMatch );
+
+static tSuccess
+Select_Equivalent( char* pzText, char* pzMatch );
+
+static tSuccess
+Select_Equivalent_End( char* pzText, char* pzMatch );
+
+static tSuccess
+Select_Equivalent_Start( char* pzText, char* pzMatch );
+
+static tSuccess
+Select_Equivalent_Full( char* pzText, char* pzMatch );
+
+static tSuccess
+Select_Match( char* pzText, char* pzMatch );
+
+static tSuccess
+Select_Match_End( char* pzText, char* pzMatch );
+
+static tSuccess
+Select_Match_Start( char* pzText, char* pzMatch );
+
+static tSuccess
+Select_Match_Full( char* pzText, char* pzMatch );
+
+static tSuccess
+Select_Match_Always( char* pzText, char* pzMatch );
+
+static tSuccess
+Select_Match_Existence( char* pzText, char* pzMatch );
+
+static tSuccess
+Select_Match_NonExistence( char* pzText, char* pzMatch );
+
+static tMacro*
+mLoad_Select( tTemplate* pT, tMacro* pMac, tCC** ppzScan );
+/* = = = END-STATIC-FORWARD = = = */
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-STATIC void
+static void
 compile_re( regex_t* pRe, char* pzPat, int flags )
 {
     int  rerr = regcomp( pRe, pzPat, flags );
@@ -74,7 +146,7 @@ compile_re( regex_t* pRe, char* pzPat, int flags )
 }
 
 
-STATIC void
+static void
 upString( char* pz )
 {
     while (*pz != NUL)
@@ -96,7 +168,7 @@ upString( char* pz )
  * doc:  Test to see if a string contains a substring.  "strstr(3)"
  *       will find an address.
 =*/
-STATIC tSuccess
+static tSuccess
 Select_Compare( char* pzText, char* pzMatch )
 {
     return (strstr( pzText, pzMatch )) ? SUCCESS : FAILURE;
@@ -125,7 +197,7 @@ ag_scm_string_contains_p( SCM text, SCM substr )
  * doc:  Test to see if a string ends with a substring.
  *       strcmp(3) returns zero for comparing the string ends.
 =*/
-STATIC tSuccess
+static tSuccess
 Select_Compare_End( char* pzText, char* pzMatch )
 {
     size_t   vlen = strlen( pzMatch );
@@ -164,7 +236,7 @@ ag_scm_string_ends_with_p( SCM text, SCM substr )
  *
  * doc:  Test to see if a string starts with a substring.
 =*/
-STATIC tSuccess
+static tSuccess
 Select_Compare_Start( char* pzText, char* pzMatch )
 {
     size_t  vlen = strlen( pzMatch );
@@ -200,7 +272,7 @@ ag_scm_string_starts_with_p( SCM text, SCM substr )
  *
  * doc:  Test to see if two strings exactly match.
 =*/
-STATIC tSuccess
+static tSuccess
 Select_Compare_Full( char* pzText, char* pzMatch )
 {
     return (strcmp( pzText, pzMatch ) == 0) ? SUCCESS : FAILURE;
@@ -232,7 +304,7 @@ ag_scm_string_equals_p( SCM text, SCM substr )
  *       to character case and certain characters are considered `equivalent'.
  *       Viz., '-', '_' and '^' are equivalent.
 =*/
-STATIC tSuccess
+static tSuccess
 Select_Equivalent( char* pzText, char* pzMatch )
 {
     char*    pz;
@@ -278,7 +350,7 @@ ag_scm_string_contains_eqv_p( SCM text, SCM substr )
  *
  * doc:  Test to see if a string ends with an equivalent string.
 =*/
-STATIC tSuccess
+static tSuccess
 Select_Equivalent_End( char* pzText, char* pzMatch )
 {
     size_t   vlen = strlen( pzMatch );
@@ -315,7 +387,7 @@ ag_scm_string_ends_eqv_p( SCM text, SCM substr )
  *
  * doc:  Test to see if a string starts with an equivalent string.
 =*/
-STATIC tSuccess
+static tSuccess
 Select_Equivalent_Start( char* pzText, char* pzMatch )
 {
     size_t   vlen = strlen( pzMatch );
@@ -355,7 +427,7 @@ ag_scm_string_starts_eqv_p( SCM text, SCM substr )
  *       This is an overloaded operation.  If the arguments are not both
  *       strings, then the query is passed through to @code{scm_num_eq_p()}.
 =*/
-STATIC tSuccess
+static tSuccess
 Select_Equivalent_Full( char* pzText, char* pzMatch )
 {
     return (streqvcmp( pzText, pzMatch ) == 0) ? SUCCESS : FAILURE;
@@ -408,7 +480,7 @@ ag_scm_string_eqv_p( SCM text, SCM substr )
  * doc:  Test to see if a string contains a pattern.
  *       Case is not significant.
 =*/
-STATIC tSuccess
+static tSuccess
 Select_Match( char* pzText, char* pzMatch )
 {
     /*
@@ -489,7 +561,7 @@ ag_scm_string_has_eqv_match_p( SCM text, SCM substr )
  * doc:  Test to see if a string ends with a pattern.
  *       Case is not significant.
 =*/
-STATIC tSuccess
+static tSuccess
 Select_Match_End( char* pzText, char* pzMatch )
 {
     regmatch_t m[2];
@@ -581,7 +653,7 @@ ag_scm_string_end_eqv_match_p( SCM text, SCM substr )
  * doc:  Test to see if a string starts with a pattern.
  *       Case is not significant.
 =*/
-STATIC tSuccess
+static tSuccess
 Select_Match_Start( char* pzText, char* pzMatch )
 {
     regmatch_t m[2];
@@ -674,7 +746,7 @@ ag_scm_string_start_eqv_match_p( SCM text, SCM substr )
  *       Case is not significant, but any character equivalences
  *       must be expressed in your regular expression.
 =*/
-STATIC tSuccess
+static tSuccess
 Select_Match_Full( char* pzText, char* pzMatch )
 {
     regmatch_t m[2];
@@ -750,14 +822,31 @@ ag_scm_string_eqv_match_p( SCM text, SCM substr )
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 /*
- *  We don't bother making a Guile function for this one :)
+ *  We don't bother making a Guile function for any of these :)
  */
-STATIC tSuccess
+static tSuccess
 Select_Match_Always( char* pzText, char* pzMatch )
 {
     return SUCCESS;
 }
 
+/*
+ *  If the "pzText" addresses "zNil", then we couldn't find a value
+ *  and defaulted to an empty string.  So, the result exists if
+ *  the pzText address is anything except "zNil".
+ */
+static tSuccess
+Select_Match_Existence( char* pzText, char* pzMatch )
+{
+    return (pzText != zDefaultNil) ? SUCCESS : FAILURE;
+}
+
+static tSuccess
+Select_Match_NonExistence( char* pzText, char* pzMatch )
+{
+    return (pzText == zDefaultNil) ? SUCCESS : FAILURE;
+}
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 /*=macfunc CASE
@@ -767,12 +856,16 @@ Select_Match_Always( char* pzText, char* pzMatch )
  *
  *  desc:
  *
- *  The arguments are evaluated and converted to a string, if necessary.
- *  (@pxref{EXPR})  The scope of the macro is up to the matching ESAC
- *  function.  Within the scope of a CASE, this string is matched against
- *  case selection macros.  There are sixteen match macros that are derived
- *  from four different ways the test may be performed, plus an "always
- *  true" match.  The code for each selection expression is formed as follows:
+ *  The arguments are evaluated and converted to a string, if necessary.  A
+ *  simple name will be interpreted as an AutoGen value name and its value will
+ *  be used by the @code{SELECT} macros (see the example below and the
+ *  expression evaluation function, @pxref{EXPR}).  The scope of the macro is
+ *  up to the matching @code{ESAC} macro.  Within the scope of a @code{CASE},
+ *  this string is matched against case selection macros.  There are sixteen
+ *  match macros that are derived from four different ways matches may be
+ *  performed, plus an "always true", "true if the AutoGen value was found",
+ *  and "true if no AutoGen value was found" matches.  The codes for the
+ *  nineteen match macros are formed as follows:
  *
  *  @enumerate
  *  @item
@@ -791,6 +884,12 @@ Select_Match_Always( char* pzText, char* pzMatch )
  *  @item
  *  Do you need a default match when none of the others match?
  *  Use a single asterisk (@code{*}).
+ *  @item
+ *  Do you need to distinguish between an empty string value and a value
+ *  that was not found?  Use the non-existence test (@code{!E}) before
+ *  testing a full match against an empty string (@code{== ''}).
+ *  There is also an existence test (@code{+E}), more for symmetry than
+ *  for practical use.
  *  @end enumerate
  *
  *  @noindent
@@ -801,6 +900,9 @@ Select_Match_Always( char* pzText, char* pzMatch )
  *  [+ ~~*  "[Tt]est" +]reg exp must match at start, not at end
  *  [+ ==   "TeSt"    +]a full-string, case sensitive compare
  *  [+ =    "TEST"    +]a full-string, case insensitive compare
+ *  [+ !E             +]not exists - matches if no AutoGen value found
+ *  [+ ==   ""        +]expression yielded a zero-length string
+ *  [+ +E             +]exists - matches if there is any value result
  *  [+ *              +]always match - no testing
  *  [+ ESAC +]
  *  @end example
@@ -810,8 +912,8 @@ Select_Match_Always( char* pzText, char* pzMatch )
  *  a number, it is converted to a decimal string.
  *
  *  These case selection codes have also been implemented as
- *  Scheme expression functions using the same codes
- *  (@pxref{Common Functions}).
+ *  Scheme expression functions using the same codes.  They are documented
+ *  in this texi doc as ``string-*?'' predicates (@pxref{Common Functions}).
 =*/
 /*=macfunc ESAC
  *
@@ -826,8 +928,9 @@ mFunc_Case( tTemplate* pT, tMacro* pMac )
 {
     typedef tSuccess (t_match_proc)( char*, char* );
     /*
-     *  There are only 13 procedures because the case insenstive matching
-     *  get mapped into the previous four.  The last is "match always".
+     *  There are only 15 procedures because the case insenstive matching
+     *  get mapped into the previous four.  The last three are "match always",
+     *  "match if a value was found" "match if no value found".
      */
     static t_match_proc* match_procs[] = {
         &Select_Compare_Full,
@@ -845,7 +948,9 @@ mFunc_Case( tTemplate* pT, tMacro* pMac )
         &Select_Match_Start,
         &Select_Match,
 
-        &Select_Match_Always
+        &Select_Match_Always,
+        &Select_Match_Existence,
+        &Select_Match_NonExistence
     };
 
     tSCC* apzMatchName[] = {
@@ -864,7 +969,9 @@ mFunc_Case( tTemplate* pT, tMacro* pMac )
         "MATCH_START",
         "MATCH_WITHIN",
 
-        "MATCH_ALWAYS"
+        "MATCH_ALWAYS",
+        "MATCH_EXISTENCE",
+        "MATCH_NONEXISTENCE"
     };
 
     tMacro*   pEnd = pT->aMacros + pMac->endIndex;
@@ -930,24 +1037,6 @@ mFunc_Case( tTemplate* pT, tMacro* pMac )
     return pEnd;
 }
 
-/*
- *  This is global data used to keep track of the current CASE
- *  statement being processed.  When CASE statements nest,
- *  these data are copied onto the stack and restored when
- *  the nested CASE statement's ESAC function is found.
- */
-typedef struct case_stack tCaseStack;
-struct case_stack {
-    tMacro*  pCase;
-    tMacro*  pSelect;
-};
-
-STATIC tCaseStack  current_case;
-STATIC tLoadProc mLoad_Select;
-
-static tpLoadProc apCaseLoad[ FUNC_CT ]   = { NULL };
-static tpLoadProc apSelectOnly[ FUNC_CT ] = { NULL };
-
 /*
  *  mLoad_CASE
  *
@@ -1050,16 +1139,17 @@ mLoad_Case( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 /*=macfunc SELECT
  *
  *  what:    Selection block for CASE function
+ *  alias:   | ~ | = | * | ! | + |
  *  desc:
  *    This macro selects a block of text by matching an expression
  *    against the sample text expression evaluated in the @code{CASE}
  *    macro.  @xref{CASE}.
  *
  *    You do not specify a @code{SELECT} macro with the word ``select''.
- *    Instead, you must use one of the 17 match operators described in
+ *    Instead, you must use one of the 19 match operators described in
  *    the @code{CASE} macro description.
 =*/
-STATIC tMacro*
+static tMacro*
 mLoad_Select( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 {
     const char*    pzScan = *ppzScan;  /* text after macro */
@@ -1120,7 +1210,26 @@ mLoad_Select( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
         }
         break;
 
+    case '!':
+    case '+':
+        switch (*pzSrc) {
+        case 'e':
+        case 'E':
+            break;
+        default:
+            goto bad_sel;
+        }
+        if ((pzSrc[1] != NUL) && (! isspace( pzSrc[1] )))
+            goto bad_sel;
+
+        typ = (pzSrc[-1] == '!')
+            ? FTYP_SELECT_MATCH_NONEXISTENCE : FTYP_SELECT_MATCH_EXISTENCE;
+        srcLen = 0;
+        pMac->ozText = 0;
+        goto selection_done;
+
     default:
+    bad_sel:
         AG_ABEND_IN( pT, pMac, zInvSel );
     }
 
