@@ -2,13 +2,13 @@
 /*
  *  tpParse.c
  *
- *  $Id: tpParse.c,v 4.1 2005/01/01 00:12:34 bkorb Exp $
+ *  $Id: tpParse.c,v 4.4 2005/06/07 22:26:44 bkorb Exp $
  *
  *  This module will load a template and return a template structure.
  */
 
 /*
- *  AutoGen copyright 1992-2004 Bruce Korb
+ *  AutoGen copyright 1992-2005 Bruce Korb
  *
  *  AutoGen is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -23,19 +23,24 @@
  *  You should have received a copy of the GNU General Public License
  *  along with AutoGen.  See the file "COPYING".  If not,
  *  write to:  The Free Software Foundation, Inc.,
- *             59 Temple Place - Suite 330,
- *             Boston,  MA  02111-1307, USA.
+ *             51 Franklin Street, Fifth Floor,
+ *             Boston, MA  02110-1301, USA.
  */
 
-STATIC teFuncType whichFunc( tCC** ppzScan );
-STATIC tCC* findMacroEnd( tCC** ppzMark );
+/* = = = START-STATIC-FORWARD = = = */
+/* static forward declarations maintained by :mkfwd */
+static teFuncType
+whichFunc( tCC** ppzScan );
 
+static tCC*
+findMacroEnd( tCC** ppzMark );
+/* = = = END-STATIC-FORWARD = = = */
 
 /*
  *  Return the enumerated function type corresponding
  *  to a name pointed to by the input argument.
  */
-STATIC teFuncType
+static teFuncType
 whichFunc( tCC** ppzScan )
 {
     const char*   pzFuncName = *ppzScan;
@@ -122,7 +127,7 @@ whichFunc( tCC** ppzScan )
 }
 
 
-STATIC tCC*
+static tCC*
 findMacroEnd( tCC** ppzMark )
 {
     tCC* pzMark = *ppzMark + startMacLen;
@@ -156,7 +161,7 @@ findMacroEnd( tCC** ppzMark )
 }
 
 
-EXPORT tMacro*
+LOCAL tMacro*
 parseTemplate( tMacro* pM, tCC** ppzText )
 {
     tCC* pzScan = *ppzText;
