@@ -1,6 +1,7 @@
 
 /*
- *  $Id: load.c,v 4.6 2005/02/04 04:24:45 bkorb Exp $
+ *  $Id: load.c,v 4.11 2005/06/07 22:26:44 bkorb Exp $
+ *  Time-stamp:      "2005-02-23 14:22:26 bkorb"
  *
  *  This file contains the routines that deal with processing text strings
  *  for options, either from a NUL-terminated string passed in or from an
@@ -23,8 +24,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with Automated Options.  See the file "COPYING".  If not,
  *  write to:  The Free Software Foundation, Inc.,
- *             59 Temple Place - Suite 330,
- *             Boston,  MA  02111-1307, USA.
+ *             51 Franklin Street, Fifth Floor,
+ *             Boston, MA  02110-1301, USA.
  *
  * As a special exception, Bruce Korb gives permission for additional
  * uses of the text contained in his release of AutoOpts.
@@ -52,12 +53,8 @@
 
 /* = = = START-STATIC-FORWARD = = = */
 /* static forward declarations maintained by :mkfwd */
-static void
-loadOptionLine(
-    tOptions*  pOpts,
-    tOptState* pOS,
-    char*      pzLine,
-    tDirection direction );
+static char*
+assembleArgValue( char* pzTxt, tOptionLoadMode mode );
 /* = = = END-STATIC-FORWARD = = = */
 
 /*=export_func  optionMakePath
@@ -238,51 +235,108 @@ optionMakePath(
 }
 
 
-static void
-loadOptionLine(
-    tOptions*  pOpts,
-    tOptState* pOS,
-    char*      pzLine,
-    tDirection direction )
+LOCAL void
+mungeString( char* pzTxt, tOptionLoadMode mode )
 {
+    char* pzE;
+
+    if (mode == OPTION_LOAD_KEEP)
+        return;
+
+    if (isspace( *pzTxt )) {
+        char* pzS = pzTxt;
+        char* pzD = pzTxt;
+        while (isspace( *++pzS ))  ;
+        while ((*(pzD++) = *(pzS++)) != NUL)   ;
+        pzE = pzD-1;
+    } else
+        pzE = pzTxt + strlen( pzTxt );
+
+    while ((pzE > pzTxt) && isspace( pzE[-1] ))  pzE--;
+    *pzE = NUL;
+
+    if (mode == OPTION_LOAD_UNCOOKED)
+        return;
+
+    switch (*pzTxt) {
+    default: return;
+    case '"':
+    case '\'': break;
+    }
+
+    switch (pzE[-1]) {
+    default: return;
+    case '"':
+    case '\'': break;
+    }
+
+    (void)ao_string_cook( pzTxt, NULL );
+}
+
+
+static char*
+assembleArgValue( char* pzTxt, tOptionLoadMode mode )
+{
+    tSCC zBrk[] = " \t:=";
+    char* pzEnd = strpbrk( pzTxt, zBrk );
+    int   space_break;
+
     /*
-     *  Strip off the first token on the line.
-     *  No quoting, space separation only.
+     *  Not having an argument to a configurable name is okay.
      */
+    if (pzEnd == NULL)
+        return pzTxt + strlen(pzTxt);
+
+    /*
+     *  If we are keeping all whitespace, then the value starts with the
+     *  character that follows the end of the configurable name, regardless
+     *  of which character caused it.
+     */
+    if (mode == OPTION_LOAD_KEEP) {
+        *(pzEnd++) = NUL;
+        return pzEnd;
+    }
+
+    /*
+     *  If the name ended on a white space character, remember that
+     *  because we'll have to skip over an immediately following ':' or '='
+     *  (and the white space following *that*).
+     */
+    space_break = isspace(*pzEnd);
+    *(pzEnd++) = NUL;
+    while (isspace(*pzEnd))  pzEnd++;
+    if (space_break && ((*pzEnd == ':') || (*pzEnd == '=')))
+        pzEnd++;
+
+    mungeString( pzEnd, mode );
+    return pzEnd;
+}
+
+
+/*
+ *  Load an option from a block of text.  The text must start with the
+ *  configurable/option name and be followed by its associated value.
+ *  That value may be processed in any of several ways.  See "tOptionLoadMode"
+ *  in autoopts.h.
+ */
+LOCAL void
+loadOptionLine(
+    tOptions*   pOpts,
+    tOptState*  pOS,
+    char*       pzLine,
+    tDirection  direction,
+    tOptionLoadMode   load_mode )
+{
+    while (isspace( *pzLine ))  pzLine++;
+
     {
-        char* pz = pzLine;
-        while (  (! isspace( *pz ))
-              && (*pz != NUL)
-              && (*pz != '=' )
-              && (*pz != ':' )  ) pz++;
+        char* pzArg = assembleArgValue( pzLine, load_mode );
 
-        /*
-         *  IF we exited because we found a character other than NUL
-         *  THEN terminate the name (clobbering our scan stop character)
-         *       and scan over any more white space that follows.
-         */
-        if (*pz != NUL) {
-            int stopped_on_space = (isspace(*pz));
-            *pz++ = NUL;
-            while (isspace(*pz))   pz++;
-
-            if (  stopped_on_space
-               && ((*pz == '=') || (*pz == ':'))  )  {
-
-                while (isspace(*(++pz)))   ;
-
-            }
-        }
-
-        /*
-         *  Make sure we can find the option in our tables and initing it is OK
-         */
         if (! SUCCESSFUL( longOptionFind( pOpts, pzLine, pOS )))
             return;
         if (pOS->flags & OPTST_NO_INIT)
             return;
-
-        pOS->pzOptArg = pz;
+        pOS->pzOptArg = pzArg;
     }
 
     switch (pOS->flags & (OPTST_IMM|OPTST_DISABLE_IMM)) {
@@ -350,145 +404,26 @@ loadOptionLine(
     /*
      *  Fix up the args.
      */
-    switch (pOS->pOD->optArgType) {
-    case ARG_NONE:
+    if (OPTST_GET_ARGTYPE(pOS->pOD->fOptState) == OPARG_TYPE_NONE) {
         if (*pOS->pzOptArg != NUL)
             return;
         pOS->pzOptArg = NULL;
-        break;
 
-    case ARG_MAY:
+    } else if (pOS->pOD->fOptState & OPTST_ARG_OPTIONAL) {
         if (*pOS->pzOptArg == NUL)
              pOS->pzOptArg = NULL;
         else AGDUPSTR( pOS->pzOptArg, pOS->pzOptArg, "option argument" );
-        break;
 
-    case ARG_MUST:
+    } else {
         if (*pOS->pzOptArg == NUL)
-             pOS->pzOptArg = "";
+             pOS->pzOptArg = zNil;
         else AGDUPSTR( pOS->pzOptArg, pOS->pzOptArg, "option argument" );
-        break;
     }
 
     handleOption( pOpts, pOS );
 }
 
 
-/*
- *  filePreset
- *
- *  Load a file containing presetting information (a configuration file).
- */
-LOCAL void
-filePreset(
-    tOptions*     pOpts,
-    const char*   pzFileName,
-    int           direction )
-{
-    typedef enum { SEC_NONE, SEC_LOOKING, SEC_PROCESS } teSec;
-    teSec   sec     = SEC_NONE;
-    FILE*   fp      = fopen( pzFileName, (const char*)"r" FOPEN_BINARY_FLAG );
-    u_int   saveOpt = pOpts->fOptSet;
-    char    zLine[ 0x1000 ];
-
-    if (fp == NULL)
-        return;
-
-    /*
-     *  DO NOT STOP ON ERRORS.  During preset, they are ignored.
-     */
-    pOpts->fOptSet &= ~OPTPROC_ERRSTOP;
-
-    /*
-     *  FOR each line in the file...
-     */
-    while (fgets( zLine, sizeof( zLine ), fp ) != NULL) {
-        char*  pzLine = zLine;
-
-        for (;;) {
-            pzLine += strlen( pzLine );
-
-            /*
-             *  IF the line is full, we stop...
-             */
-            if (pzLine >= zLine + (sizeof( zLine )-2))
-                break;
-            /*
-             *  Trim of trailing white space.
-             */
-            while ((pzLine > zLine) && isspace(pzLine[-1])) pzLine--;
-            *pzLine = NUL;
-            /*
-             *  IF the line is not continued, then exit the loop
-             */
-            if (pzLine[-1] != '\\')
-                break;
-            /*
-             *  insert a newline and get the continuation
-             */
-            pzLine[-1] = '\n';
-            fgets( pzLine, sizeof( zLine ) - (int)(pzLine - zLine), fp );
-        }
-
-        pzLine = zLine;
-        while (isspace( *pzLine )) pzLine++;
-
-        switch (*pzLine) {
-        case NUL:
-        case '#':
-            /*
-             *  Ignore blank and comment lines
-             */
-            continue;
-
-        case '[':
-            /*
-             *  Enter a section IFF sections are requested and the section
-             *  name matches.  If the file is not sectioned,
-             *  then all will be handled.
-             */
-            if (pOpts->pzPROGNAME == NULL)
-                goto fileDone;
-
-            switch (sec) {
-            case SEC_NONE:
-                sec = SEC_LOOKING;
-                /* FALLTHROUGH */
-
-            case SEC_LOOKING:
-            {
-                int secNameLen = strlen( pOpts->pzPROGNAME );
-                if (  (strncmp( pzLine+1, pOpts->pzPROGNAME, secNameLen ) != 0)
-                   || (pzLine[secNameLen+1] != ']')  )
-                    continue;
-                sec = SEC_PROCESS;
-                break;
-            }
-
-            case SEC_PROCESS:
-                goto fileDone;
-            }
-            break;
-
-        default:
-            /*
-             *  Load the line only if we are not in looking-for-section state
-             */
-            if (sec == SEC_LOOKING)
-                continue;
-        }
-
-        {
-            tOptState st = { NULL, OPTST_PRESET, TOPT_UNDEFINED, 0, NULL };
-            loadOptionLine( pOpts, &st, pzLine, direction );
-        }
-    } fileDone:;
-
-    pOpts->fOptSet = saveOpt;
-    fclose( fp );
-}
-
-
 /*=export_func  optionLoadLine
  *
  * what:  process a string for an option name and value
@@ -498,15 +433,18 @@ filePreset(
  *
  * doc:
  *
- * This is a client program callable routine for setting options from, for
- * example, the contents of a file that they read in.  Only one option may
- * appear in the text.  It will be treated as a normal (non-preset) option.
+ *  This is a client program callable routine for setting options from, for
+ *  example, the contents of a file that they read in.  Only one option may
+ *  appear in the text.  It will be treated as a normal (non-preset) option.
  *
- * When passed a pointer to the option struct and a string, it will
- * find the option named by the first token on the string and set
- * the option argument to the remainder of the string.  The caller must
- * NUL terminate the string.  Any embedded new lines will be included
- * in the option argument.
+ *  When passed a pointer to the option struct and a string, it will find
+ *  the option named by the first token on the string and set the option
+ *  argument to the remainder of the string.  The caller must NUL terminate
+ *  the string.  Any embedded new lines will be included in the option
+ *  argument.  If the input looks like one or more quoted strings, then the
+ *  input will be "cooked".  The "cooking" is identical to the string
+ *  formation used in AutoGen definition files (@pxref{basic expression}),
+ *  except that you may not use backquotes.
  *
  * err:   Invalid options are silently ignored.  Invalid option arguments
  *        will cause a warning to print, but the function should return.
@@ -516,68 +454,17 @@ optionLoadLine(
     tOptions*  pOpts,
     tCC*       pzLine )
 {
-    tOptState st = { NULL, OPTST_SET, TOPT_UNDEFINED, 0, NULL };
+    tOptState st = OPTSTATE_INITIALIZER(SET);
     char* pz;
     AGDUPSTR( pz, pzLine, "user option line" );
-    loadOptionLine( pOpts, &st, pz, DIRECTION_PROCESS );
+    loadOptionLine( pOpts, &st, pz, DIRECTION_PROCESS, OPTION_LOAD_COOKED );
     AGFREE( pz );
 }
-
-
-/*=export_func  doLoadOpt
- * private:
- *
- * what:  Load an option rc/ini file
- * arg:   + tOptions* + pOpts    + program options descriptor +
- * arg:   + tOptDesc* + pOptDesc + the descriptor for this arg +
- *
- * doc:
- *  Processes the options found in the file named with pOptDesc->pzLastArg.
-=*/
-void
-doLoadOpt( tOptions* pOpts, tOptDesc* pOptDesc )
-{
-    /*
-     *  IF the option is not being disabled,
-     *  THEN load the file.  There must be a file.
-     *  (If it is being disabled, then the disablement processing
-     *  already took place.  It must be done to suppress preloading
-     *  of ini/rc files.)
-     */
-    if (! DISABLED_OPT( pOptDesc )) {
-        struct stat sb;
-        if (stat( pOptDesc->pzLastArg, &sb ) != 0) {
-            tSCC zMsg[] =
-                "File error %d (%s) opening %s for loading options\n";
-
-            if ((pOpts->fOptSet & OPTPROC_ERRSTOP) == 0)
-                return;
-
-            fprintf( stderr, zMsg, errno, strerror( errno ),
-                     pOptDesc->pzLastArg );
-            (*pOpts->pUsageProc)( pOpts, EXIT_FAILURE );
-            /* NOT REACHED */
-        }
-
-        if (! S_ISREG( sb.st_mode )) {
-            if ((pOpts->fOptSet & OPTPROC_ERRSTOP) == 0)
-                return;
-
-            fprintf( stderr, zNotFile, pOptDesc->pzLastArg );
-            (*pOpts->pUsageProc)( pOpts, EXIT_FAILURE );
-            /* NOT REACHED */
-        }
-
-        filePreset( pOpts, pOptDesc->pzLastArg, DIRECTION_PROCESS );
-    }
-}
-
 /*
  * Local Variables:
  * mode: C
  * c-file-style: "stroustrup"
  * tab-width: 4
  * indent-tabs-mode: nil
- * tab-width: 4
  * End:
  * end of autoopts/load.c */
