@@ -1,6 +1,7 @@
 
 /*
- *  save.c  $Id: save.c,v 4.8 2005/02/04 04:24:45 bkorb Exp $
+ *  save.c  $Id: save.c,v 4.11 2005/06/07 22:26:44 bkorb Exp $
+ * Time-stamp:      "2005-02-20 13:49:46 bkorb"
  *
  *  This module's routines will take the currently set options and
  *  store them into an ".rc" file for re-interpretation the next
@@ -23,8 +24,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with Automated Options.  See the file "COPYING".  If not,
  *  write to:  The Free Software Foundation, Inc.,
- *             59 Temple Place - Suite 330,
- *             Boston,  MA  02111-1307, USA.
+ *             51 Franklin Street, Fifth Floor,
+ *             Boston, MA  02110-1301, USA.
  *
  * As a special exception, Bruce Korb gives permission for additional
  * uses of the text contained in his release of AutoOpts.
@@ -55,10 +56,10 @@ tSCC  zWarn[] = "%s WARNING:  cannot save options - ";
 /* = = = START-STATIC-FORWARD = = = */
 /* static forward declarations maintained by :mkfwd */
 static tCC*
-findDirName( tOptions* pOpts, int* p_free_res );
+findDirName( tOptions* pOpts, int* p_free );
 
 static tCC*
-findFileName( tOptions* pOpts, int* p_free_res );
+findFileName( tOptions* pOpts, int* p_free_name );
 
 static void
 printEntry(
@@ -283,7 +284,7 @@ printEntry(
      *  IF the option is numeric only,
      *  THEN the char pointer is really the number
      */
-    if ((p->fOptState & OPTST_NUMERIC) != 0)
+    if (OPTST_GET_ARGTYPE(p->fOptState) == OPARG_TYPE_NUMERIC)
         fprintf( fp, "  %d\n", (t_word)pzLA );
 
     /*
@@ -342,9 +343,6 @@ printEntry(
  * and do nothing.  If the output file cannot be created or updated, a message
  * will be printed to @code{stderr} and the routine will return.
 =*/
-#define ARGTYPE (OPTST_NUMERIC \
-  | OPTST_STACKED | OPTST_ENUMERATION | OPTST_MEMBER_BITS | OPTST_BOOLEAN )
-
 void
 optionSaveFile( tOptions* pOpts )
 {
@@ -429,38 +427,40 @@ optionSaveFile( tOptions* pOpts )
          *  IF    no arguments are allowed
          *  THEN just print the name and continue
          */
-        if (p->optArgType == ARG_NONE) {
+        if (OPTST_GET_ARGTYPE(pOD->fOptState) == OPARG_TYPE_NONE) {
             fprintf( fp, "%s\n",
                      (DISABLED_OPT( p )) ? p->pz_DisableName : p->pz_Name );
             continue;
         }
 
-        arg_state = p->fOptState & ARGTYPE;
+        arg_state = OPTST_GET_ARGTYPE(p->fOptState);
         switch (arg_state) {
         case 0:
-        case OPTST_NUMERIC:
+        case OPARG_TYPE_NUMERIC:
             printEntry( fp, p, p->pzLastArg );
             break;
 
-        case OPTST_STACKED:
-        {
-            tArgList*  pAL = (tArgList*)p->optCookie;
-            int        uct = pAL->useCt;
-            tCC**      ppz = pAL->apzArgs;
+        case OPARG_TYPE_STRING:
+            if (p->fOptState & OPTST_STACKED) {
+                tArgList*  pAL = (tArgList*)p->optCookie;
+                int        uct = pAL->useCt;
+                tCC**      ppz = pAL->apzArgs;
 
-            /*
-             *  Disallow multiple copies of disabled options.
-             */
-            if (uct > 1)
-                p->fOptState &= ~OPTST_DISABLED;
+                /*
+                 *  Disallow multiple copies of disabled options.
+                 */
+                if (uct > 1)
+                    p->fOptState &= ~OPTST_DISABLED;
 
-            while (uct-- > 0)
-                printEntry( fp, p, *(ppz++) );
+                while (uct-- > 0)
+                    printEntry( fp, p, *(ppz++) );
+            } else {
+                printEntry( fp, p, p->pzLastArg );
+            }
             break;
-        }
 
-        case OPTST_ENUMERATION:
-        case OPTST_MEMBER_BITS:
+        case OPARG_TYPE_ENUMERATION:
+        case OPARG_TYPE_MEMBERSHIP:
         {
             tCC* val = p->pzLastArg;
             /*
@@ -469,7 +469,7 @@ optionSaveFile( tOptions* pOpts )
              */
             (*(p->pOptProc))( (tOptions*)2UL, p );
             printEntry( fp, p, p->pzLastArg );
-            if ((p->pzLastArg != NULL) && (arg_state != OPTST_ENUMERATION))
+            if ((p->pzLastArg != NULL) && (arg_state != OPARG_TYPE_ENUMERATION))
                 /*
                  *  bit flag and enumeration strings get allocated
                  */
@@ -478,12 +478,12 @@ optionSaveFile( tOptions* pOpts )
             break;
         }
 
-        case OPTST_BOOLEAN:
+        case OPARG_TYPE_BOOLEAN:
             printEntry( fp, p, (p->pzLastArg != 0) ? "true" : "false" );
             break;
 
         default:
-            break; /* bogus - skip it */
+            break; /* cannot handle - skip it */
         }
     } while ( (pOD++), (--ct > 0));
 
